@@ -418,3 +418,45 @@ TEST_CASE("PadamLogisticRegressionTest", "[AdamTest]")
       coordinates);
   REQUIRE(testAcc == Approx(100.0).epsilon(0.006)); // 0.6% error tolerance.
 }
+
+/**
+ * Tests the QHadam optimizer using a simple test function.
+ */
+TEST_CASE("SimpleQHAdamTestFunction", "[AdamTest]")
+{
+  SGDTestFunction f;
+  QHAdam optimizer(0.02, 32, 0.6, 0.9, 0.9, 0.999, 1e-8, 200000, 1e-7, true);
+
+  arma::mat coordinates = f.GetInitialPoint();
+  optimizer.Optimize(f, coordinates);
+
+  bool success = (coordinates[0] == Approx(0.0).margin(0.3)) &&
+                 (coordinates[1] == Approx(0.0).margin(0.3)) &&
+                 (coordinates[2] == Approx(0.0).margin(0.3));
+  REQUIRE(success == true);
+}
+
+/**
+ * Run QHAdam on logistic regression and make sure the results are acceptable.
+ */
+TEST_CASE("QHAdamLogisticRegressionTest", "[AdamTest]")
+{
+  arma::mat data, testData, shuffledData;
+  arma::Row<size_t> responses, testResponses, shuffledResponses;
+
+  LogisticRegressionTestData(data, testData, shuffledData,
+      responses, testResponses, shuffledResponses);
+  LogisticRegression<> lr(shuffledData, shuffledResponses, 0.5);
+
+  QHAdam optimizer;
+  arma::mat coordinates = lr.GetInitialPoint();
+  optimizer.Optimize(lr, coordinates);
+
+  // Ensure that the error is close to zero.
+  const double acc = lr.ComputeAccuracy(data, responses, coordinates);
+  REQUIRE(acc == Approx(100.0).epsilon(0.003)); // 0.3% error tolerance.
+
+  const double testAcc = lr.ComputeAccuracy(testData, testResponses,
+      coordinates);
+  REQUIRE(testAcc == Approx(100.0).epsilon(0.006)); // 0.6% error tolerance.
+}
